/*
 * @Author: 祝玲云
 * @Date: 2021-06-25 14:01:20
 * @LastEditTime: 2021-06-25 14:01:20
 * @LastEditors: Please set LastEditors
 * @Description: 插件统一入口
 * @FilePath: \xs-plugins\src\lib\index.js
 */

const path = require('path')
const RouterPlugins = require('./router')
const ApiPlugins = require('./api')
const MenuPlugin = require('./menu')
const fs = require('fs');

// 自动生成插件默认配置项
const plugins = {
    autoPath: 'src/automatically',  // 文件生成位置
    entry: '', // 遍历文件夹的位置
}

class IndexPlugins {
    constructor(options) {
        plugins.entry = path.join('/src/modules/' + options.moduleConfig.CUR_MODULES + '/')
        this.options = options
    }
    // 自动化插件统一入口
    async apply (compiler) {
        if(process.env.NODE_ENV === 'development') {
            this.getOptions(compiler)
        } else {
            // 删除自动生成文件
            this.delDir(path.join(this.options.rootPath, '/' + plugins.autoPath))
            compiler.plugin('run', async (compilation, callback) => { // 将事件添加到线程中， 避免文件还未生成， 就执行了其他插件
                await this.getOptions (compiler)
                callback()
            })
        }
    }
    async getOptions (compiler){
        /**
         * 当后期 api 自动生成 优化为 开发环境生成， 生成环境不生成时， api.js 中的 获取 client.js 部分推荐封装成处理， 以便 menu.js 中使用
         */
        let router = new RouterPlugins({...plugins, ...this.options}).apply(compiler)
        let apiProcess =  new ApiPlugins({...plugins, ...this.options}) // api 进程
        await apiProcess.apply(compiler);
        await new MenuPlugin({...plugins, ...this.options, router, api: apiProcess.getApi(), clientInfo: apiProcess.getClientInfo()}).apply(compiler)
    }
    delDir (path){
        let files = [];
        if(fs.existsSync(path)){
            files = fs.readdirSync(path);
            files.forEach((file, index) => {
                let curPath = path + "/" + file;
                if(fs.statSync(curPath).isDirectory()){
                    this.delDir(curPath); //递归删除文件夹
                } else {
                    fs.unlinkSync(curPath); //删除文件
                }
            });
            fs.rmdirSync(path);
        }
    }
}

module.exports = IndexPlugins